
from osv import osv, fields
from tools.translate import _
import os
import base64, urllib
import netsvc

from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import time



class Region(osv.osv):

    _description="Region"
    _name = 'event.region'
    

    _columns = {
    
        'name': fields.char('Region Name', size=64, required=True),
        'code': fields.char('Region Code', size=3),
    }
 
    _sql_constraints = [
        ('name_uniq', 'unique (name)',
            'The name of the region must be unique !'),
        ('code_uniq', 'unique (code)',
            'The code of the region must be unique !')
    ]

    def name_search(self, cr, user, name='', args=None, operator='ilike',
            context=None, limit=100):
        if not args:
            args=[]
        if not context:
            context={}
        ids = False
        if len(name) == 2:
            ids = self.search(cr, user, [('code', 'ilike', name)] + args,limit=limit, context=context)
        if not ids:
            ids = self.search(cr, user, [('name', operator, name)] + args,limit=limit, context=context)
        return self.name_get(cr, user, ids, context)
    
    def create(self, cursor, user, vals, context=None):
        if 'code' in vals:
            vals['code'] = vals['code'].upper()
        return super(Region, self).create(cursor, user, vals,context=context)

    def write(self, cursor, user, ids, vals, context=None):
        if 'code' in vals:
            vals['code'] = vals['code'].upper()
        return super(Region, self).write(cursor, user, ids, vals,context=context)
        
    _order='name'
    

Region()

class City(osv.osv):
    
    _name = 'event.city'
    
    
    _columns = {
                    'region_id': fields.many2one('event.region', 'Region',required=True),
                    'name': fields.char('City Name', size=64, required=True),
                    'code': fields.char('City Code', size=3),
                }
    
    def name_search(self, cr, user, name='', args=None, operator='ilike',
            context=None, limit=100):
        if not args:
            args = []
        if not context:
            context = {}
        ids = self.search(cr, user, [('code', 'ilike', name)] + args, limit=limit,
                context=context)
        if not ids:
            ids = self.search(cr, user, [('name', operator, name)] + args,
                    limit=limit, context=context)
        return self.name_get(cr, user, ids, context)

    
    _order = 'code'

City()


class Classification(osv.osv):
    def name_get(self, cr, uid, ids, context=None):
        if not len(ids):
            return []
        reads = self.read(cr, uid, ids, ['name','parent_id'], context=context)
        res = []
        for record in reads:
            name = record['name']
            if record['parent_id']:
                pass
                name = record['parent_id'][1]+' / '+name
            res.append((record['id'], name))
        return res

    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=100):
        if not args:
            args=[]
        if not context:
            context={}
        if name:
            # Be sure name_search is symetric to name_get
            name = name.split(' / ')[-1]
            ids = self.search(cr, uid, [('name', operator, name)] + args, limit=limit, context=context)
        else:
            ids = self.search(cr, uid, args, limit=limit, context=context)
        return self.name_get(cr, uid, ids, context)


    def _name_get_fnc(self, cr, uid, ids, prop, unknow_none, context=None):
        res = self.name_get(cr, uid, ids, context=context)
        return dict(res)

    _description='Classification'
    _name = 'event.classification'
    
    _columns = {
        'name': fields.char('Category Name', required=True, size=64, translate=True),
        'parent_id': fields.many2one('event.classification', 'Parent Category', select=True, ondelete='cascade'),
        'complete_name': fields.function(_name_get_fnc, method=True, type="char", string='Full Name'),
        'child_ids': fields.one2many('event.classification', 'parent_id', 'Child Categories'),
        'active' : fields.boolean('Active', help="The active field allows you to hide the category without removing it."),
        'parent_left' : fields.integer('Left parent', select=True),
        'parent_right' : fields.integer('Right parent', select=True),
    }
    _constraints = [
        (osv.osv._check_recursion, 'Error ! You can not create recursive categories.', ['parent_id'])
    ]
    _defaults = {
        'active' : lambda *a: 1,
    }
    _parent_store = True
    _parent_order = 'name'
    _order = 'parent_left'
    

Classification()

class Package(osv.osv):
    
   
    _name="event.package"
    
    _columns={
              'name':fields.char('Name',size=128,required=True),
         
              'product_ids':fields.one2many('event.product','package_id','Products'),
              'main_class':fields.many2one('event.classification','Main clasification',required=True),
              'theme':fields.many2one('event.classification','Theme'),
              'sub_theme':fields.many2one('event.classification','Sub Theme'),
             
              }

Package()   

class Service(osv.osv):
    
    _name="event.service"
    
Service()     

class EventProduct(osv.osv):
    
    _name="event.product"
    
EventProduct()        

   

class res_partner(osv.osv):
    
    _name = "res.partner"
    
    _inherit = "res.partner"
    
        
    _columns={
              'main_class':fields.many2one('event.classification','Main clasification'),
              'theme':fields.many2one('event.classification','Theme'),
              'sub_theme':fields.many2one('event.classification','Sub Theme'),
              'service_ids':fields.many2many('event.service','french_service_partner_rel','service_ids','partner_id','Services'),  
              'product_ids':fields.many2many('event.product','french_product_partner_rel','product_ids','partner_id','Products'),
              'last_update':fields.date('Last Update'),
              'bank_id':fields.many2one('res.bank',"Bank"),
              'activity':fields.many2one('event.activity','Activity'),
              'tovisit':fields.many2one('event.tovisit','To visit'),
              'contact_user':fields.many2one('event.contact','Contact found by'),
              'vat_num':fields.char('VAT number',size=32),
              'grp_name':fields.char('Group name',size=64),
              }
    
   
    
    _defaults={
               'supplier': lambda *a :1,
               'customer': lambda *a :0,
              
              }
  
    
res_partner()

class EventProductImages(osv.osv):
    
    _name="event.images"
    
    def get_image(self, cr, uid, ids):
        each = self.read(cr, uid, ids, ['link','image','url'])
        
        try:
            if each['link'] and each['url']:
                (filename, header) = urllib.urlretrieve(each['url'])
                f = open(filename , 'rb')
                img = base64.encodestring(f.read())
                f.close()
            elif each['image']:
                img = each['image']
            else:
                img = False
            
        except:
            img = False
            raise osv.except_osv(_('Warning'), _('Image url is not valid, please check the url.'))
           
        return img
        
    
    def _get_image(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for each in ids:
            img = self.get_image(cr, uid, each)
            res[each] = img
        return res
    
    _columns={
              'name':fields.char('Name',size=64,required=True),
              'url':fields.char('Url',size=255),
              'link':fields.boolean('Link'),
              'preview':fields.function(_get_image,string='Image',type='binary',method=True),
              'image':fields.binary('Image'),
              'product_id':fields.many2one('event.product','Products'),
              'service_id':fields.many2one('event.service','Services'),
              }
    
EventProductImages()    

class Service(osv.osv):
    
    _name = 'event.service'
    
    
    def _get_selling_price(self,cr,uid,ids,name,args,context):
        
        res={}
        
        for id in self.browse(cr,uid,ids):            
            margin=id.margin_coeff.value or 0.0
            unit_buy_value=id.unit_buy or 0.0
#            selling_value=unit_buy_value*100/(margin or 1.0)
            selling_value=((unit_buy_value*margin)/100)+unit_buy_value
                
        res[ids[0]]=selling_value       
       
        return res
                          
    
    _columns={
              'name':fields.char('Service name', size=128, required=True),
              'vat':fields.many2one('event.vat','Vat',required=True),
              'region_id':fields.many2one('event.region','Region',required=True),
              'city_id':fields.many2one("event.city", 'City',required=True,domain="[('region_id','=',region_id)]"),
              'description':fields.text('Description'),
              'unit_cap':fields.integer('Unit Capacity',required=True),
              'min_cap':fields.integer('Minimun Capacity',required=True),
              'max_cap':fields.integer('Max Capacity',required=True),
              'unit_buy':fields.float('Unit buying price', digits=(16,2),required=True),
              'margin_coeff':fields.many2one('event.margin.coeff','Margin coefficient',required=True),
              'sale_price':fields.function(_get_selling_price,string='Selling price',method=True,store=True,digits=(16,2)),
              'product_id':fields.many2one('event.product','Product'),
              'partner_id':fields.many2one('res.partner','Supplier'),
              'main_class':fields.many2one('event.classification','Main classification',required=True),
              'theme':fields.many2one('event.classification','Theme'),
              'sub_theme':fields.many2one('event.classification','Sub Theme'),
#              'image_ids':fields.one2many('event.images','service_id','Images'),
#              'image_ids':fields.many2many('event.images','service_image_rel','image','service','Images'),  
              }
    
    _defaults={
               'unit_cap':lambda *a:1,
               'min_cap':lambda *a:1,
               'max_cap':lambda *a:100,
               'unit_buy': lambda *a : 1,
                   
               }
    
    def create_product(self,cr,uid,ids,context=None):
        
        for id in self.browse(cr,uid,ids):
            pro_name=id.name
#            if not id.partner_id.id:
#                raise osv.except_osv(_('Warning'), _('Please provide supplier name!.'))
            partner_id=id.partner_id.id
            main_class_id=id.main_class.id
            theme_id=(id.theme or '') and id.theme.id
            sub_theme=(id.sub_theme or '') and id.sub_theme.id
            region_id=(id.region_id or '') and id.region_id.id
            city_id=(id.city_id or '') and id.city_id.id
        
        product_obj=self.pool.get('event.product')
        data={
                'name':pro_name,
                'partner_id': partner_id,
                'sale_price': 1,
                'cost_price': 1,
                'main_class':main_class_id,              
                'theme':theme_id,              
                'sub_theme':sub_theme,        
                'region_id':region_id,
                'city_id':city_id,                             
              }
        
        product_obj.create(cr,uid,data)
        
        return True

Service()



class res_partner_address(osv.osv):
    
    _name='res.partner.address'
    _inherit='res.partner.address'
    
    _columns={
              'website':fields.char('website',size=64),
              'skype':fields.char('skype',size=64),
              }
    
    
res_partner_address()




class EventProduct(osv.osv):
    
    _name="event.product"
    
    _columns={
              'name':fields.char('Product',size=128,required=True),
              'partner_id':fields.many2one('res.partner','Supplier'),
              'min_cap':fields.integer('Minimum Capacity'),
              'description':fields.text('Description'),
              'max_cap':fields.integer('Maximum Capacity'),
              'cost_price':fields.float('Cost Price',digits=(16,2),required=True),
              'sale_price':fields.float('Sale Price',digits=(16,2),required=True),
              'region_id':fields.many2one('event.region','Region'),
              'city_id':fields.many2one("event.city", 'City', domain="[('region_id','=',region_id)]"),
              'responsible_id':fields.many2one('res.users','Responsible',help="This is use as task responsible"),
              'service_quan_ids':fields.one2many('product.service','product_service_id','Service Quatity'),
#              'image_ids':fields.one2many('event.images','product_id','Images'),
              'image_ids':fields.many2many('event.images','service_image_rel','image','service','Images'),  
              'main_class':fields.many2one('event.classification','Main clasification',required=True),
              'theme':fields.many2one('event.classification','Theme'),
              'sub_theme':fields.many2one('event.classification','Sub Theme'),
              'package_id':fields.many2one('event.package','Package'),
              }
    
        
    
    _sql_constraints=[('unique name','unique(name)','Product name should be unique!')]
    
    

    def onchange_service(self,cr,uid,ids,service_ids,context=None):
        
        res={}
      
        min_cap_list=[] 
        max_cap_list=[]    
        sp_list=[]
        cp_list=[]
        
        if not service_ids:
            
            res={'sale_price':0,'cost_price':0,'min_cap':0,'max_cap':0}
            return {'value':res}
        
        for obj in service_ids:
            min_cap_list.append(obj[2]['min_cap'])
            max_cap_list.append(obj[2]['max_cap'])
            sp_list.append(obj[2]['total'])
            cp_list.append(obj[2]['unit_buy']*obj[2]['quantity'])

        res={'sale_price':sum(sp_list),'cost_price':sum(cp_list),'min_cap':max(min_cap_list or [0]),'max_cap':min(max_cap_list or [0])}
            
        return {'value':res}
    
EventProduct() 

class ProductService(osv.osv):
    
    _name="product.service"
    
    
    def _get_total(self,cr,uid,ids,field,args,context):
        res={}
        total=0.0
        if ids:
            for each in self.browse(cr,uid,ids,context=context):
                total=each.sale_price*each.quantity
                res[each.id]=total
        return res        
    
    def onchange_service(self,cr,uid,ids,service_id,context=None):

        res={}
        
        if service_id:
            
            for each in self.pool.get('event.service').browse(cr,uid,[service_id],context=context):
                
                
                res['min_cap']=each.min_cap
                res['max_cap']=each.max_cap
                res['sale_price']=each.sale_price
                res['unit_buy']=each.unit_buy
                res['vat'] = each.vat and each.vat.name or ''
                          
                  
        return {'value':res}
    
    _columns={

              'product_service_id':fields.many2one('event.product','Products'),
              'unit_cap':fields.integer('Unit capacity'),
              'quantity':fields.integer('Quantity'),
              'service_id':fields.many2one('event.service','Service'),
              'min_cap':fields.integer('Minimum Capacity'),
              'max_cap':fields.integer('Maximum Capacity'),
              'vat':fields.char('VAT',size=64),
              'partner_id':fields.related('service_id','partner_id',type='many2one',relation='res.partner',string="Supplier"),
              'sale_price':fields.float('Selling Price',digits=(16,2)),
              'unit_buy':fields.integer('Unit buying price'),
              'total': fields.function(_get_total,digits=(16,2),type='float',string="Total",store=True,method=True),  

              }
    
    _defaults={
               'quantity': lambda *a : 1,
               'unit_cap': lambda *a : 1,
               }
    
ProductService()





















