

from osv import osv, fields
from tools.translate import _
import os
import base64, urllib
import netsvc

from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
import time
import cStringIO
import xlwt




class EventOrder(osv.osv):
    
    UNIT_SELECTION=[('day','Days'),('hr','Hrs'),('min','Mins')]
    
    
    
    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        
        cur_obj = self.pool.get('res.currency')
        res = {}
        for order in self.browse(cr, uid, ids, context=context):
            res[order.id] = {'amount_untaxed': 0.0,'amount_tax': 0.0,'amount_total': 0.0,'total_buy':0.0}
            
            for line in order.order_line:
               res[order.id]['amount_tax'] += round(line.amount_taxed,3)
               res[order.id]['amount_untaxed']+= round(line.amount_untaxed,3)
               res[order.id]['amount_total'] += round(line.amount_total,3)
               res[order.id]['total_buy']+= round(line.price_unit*line.quantity,3) 
       
            
        return res
    
    def button_dummy(self, cr, uid, ids, context=None):
        
        order_object=self.pool.get('event.order.line')
        date_list=[]
        res={}
        for event in self.browse(cr,uid,ids):
            
            for order in event.order_line:
                date_list.append(order.day)
          
            self.write(cr,uid,ids,{'start_date':min(date_list),'end_date':max(date_list)},context=context)    
            
        return True

    def partner_id_change(self,cr,uid,ids,partner_id=False,context=None):
        
        if not  partner_id:
            raise osv.except_osv(_('No Customer Defined !'), _('You have to select a customer in the sales form !\nPlease set one customer before choosing a product.'))
  
        if partner_id:
            partner_object=self.pool.get('res.partner')
            lang = partner_object.browse(cr, uid, partner_id).lang
            context = {'lang': lang, 'partner_id': partner_id}
            
        result = {'main_class':False,'theme':False,'sub_theme':False}
             
        return {'value': result}
    
    def onchange_package(self,cr,uid,package_id=None,context=None):
        result={}
        
        if not package_id:
            return False
                
        package_obj=self.pool.get('event.package')
        for package in package_obj.browse(cr,uid,package_id):
           
            result['main_class']=package.main_class and package.main_class.name or False
            result['sub_theme']=package.main_class and package.sub_theme.name or False
            result['theme']=package.main_class and package.theme.name or False

        return {'value':result}

    def _get_total_time(self,cr,uid,ids,name,args,context):
        
        res={}
        for record in self.browse(cr,uid,ids):
            quot_time=record.quot_creation or 0
            tracking=record.tr_and_ed or 0
            project=record.on_project or 0
            total_time=quot_time+tracking+project
        
        res[ids[0]]=total_time
        return  res
     
   

    _name="event.quotation"
    
    _inherit="mailgate.thread"
    
    _columns={
              'name':fields.char('Quote on file',size=32,readonly=True),
              'invoice':fields.char('Invoice No',size=32,readonly=True),
             
#              'package_id':fields.many2one('event.package','Package name',required=True),
              'active':fields.boolean('Active'),
              'operation_id':fields.many2one('event.operation','Kind of operation'),
              'cust_id':fields.many2one('res.partner','Customer',required=True),
              'main_class':fields.many2one('event.classification','Main clasification'),
              'theme':fields.many2one('event.classification','Theme'),
              'sub_theme':fields.many2one('event.classification','Sub Theme'),
              'user_id':fields.many2one('res.users','Responsible'),
              'start_date':fields.date('Starting date'),
              'win_lost':fields.many2one('event.winlost','Bussiness status'),
              'participant':fields.integer('Participant'),
              'comm_pay':fields.integer('Commission to pay',states={'invoiced':[('required',True)],'done':[('required',True)]}),
              'comm_rec':fields.integer('Commission to receive',states={'invoiced':[('required',True)],'done':[('required',True)]}),
              'end_date':fields.date('Ending date'),
              'region_id':fields.many2one('event.region','Region'),
              'city_id':fields.many2one("event.city", 'City',required=True,domain="[('region_id','=',region_id)]"),
              'invoiced':fields.boolean('Invoiced',readonly=True),
              'amount_untaxed': fields.function(_amount_all,digits=(16,3) ,method=True, string='Untaxed Amount',store = True, multi='sums', help="The amount without tax."),
              'amount_tax': fields.function(_amount_all,digits=(16,3),method=True, string='Taxes',store = True, multi='sums', help="The tax amount."),
              'amount_total': fields.function(_amount_all,digits=(16,3) ,method=True, string='Total',store = True, multi='sums', help="The total amount."),
              'state':fields.selection([
                                        ('draft','Draft'),
                                        ('proforma','Proforma invoice sent'),
                                        ('invoiced','Invoice sent'),
                                        ('done','Done'),
                                        ('quotation','Quotation in progress'),
                                        ('cancel','Cancelled'),
                                        ],'State',readonly=True),
              'recp_date':fields.date('Reception date',required=True),
              'quot_date':fields.date('Quotation sent'),
              'last_recall':fields.date('Last recall'),
              'next_recall':fields.date('Next recall'),
              'budget':fields.integer('Budget'),
              'date_cancel':fields.date('Cancellation date'),
              'quot_creation':fields.integer('Quotation creation'),
              'tr_and_ed':fields.integer('Tracking and editing'),
              'on_project':fields.integer('On project'),
              'notes_gen':fields.text('Notes'),
              'notes_sol':fields.text('Notes'),
              'notes_can':fields.text('Notes'),
              'total_time':fields.function(_get_total_time,string='Total time',method=True,store=True),
              'order_line':fields.one2many('event.order.line','order_id','Order Line'),
              'time_unit':fields.selection(UNIT_SELECTION,'Time unit'),
              'total_buy':fields.function(_amount_all,digits=(16,3),multi='sums',method=True,store=True),

              }
    
    _defaults={
               'comm_pay': lambda *a: False,
               'state': lambda *a:'draft',
               'active': lambda *a:True, 
               'user_id': lambda self,cr,uid,ids:uid,
               'time_unit':'hr',
               }
    
    
    def to_draft(self,cr,uid,ids,context=None):
        
        self.write(cr,uid,ids,{'state':'draft','invoiced':False})
        state=self.browse(cr,uid,ids,context=context)[0].state
        objs=self.browse(cr,uid,ids)
        self._history(cr, uid, objs, _(state or ''))
        self.log(cr, uid,ids[0], "Quotation set to draft!")
        
        return True
    
    def to_quotation(self,cr,uid,ids,context=None):
       
        self.write(cr,uid,ids,{'state':'quotation'})
        state=self.browse(cr,uid,ids,context=context)[0].state
        objs=self.browse(cr,uid,ids)
        self._history(cr, uid, objs, _(state or ''))
        self.log(cr, uid,ids[0], "Quotation sent!")
        
        return True
    
    def to_proforma(self,cr,uid,ids,context=None):
        
        self.write(cr,uid,ids,{'state':'proforma','invoiced':False})
        state=self.browse(cr,uid,ids,context=context)[0].state
        objs=self.browse(cr,uid,ids)
        self._history(cr, uid, objs, _(state or ''))
        self.log(cr, uid,ids[0], "Quotation in proforma!")
        
        return True
    
    def to_cancel(self,cr,uid,ids,context=None):
        
        self.write(cr,uid,ids,{'state':'cancel','invoiced':False})
        state=self.browse(cr,uid,ids,context=context)[0].state
        objs=self.browse(cr,uid,ids)
        self._history(cr, uid, objs, _(state or ''))
        self.log(cr, uid,ids[0], "Quotation cancelled!")
        
        return True
    
    def to_invoiced(self,cr,uid,ids,context=None):
        
        self.write(cr,uid,ids,{'state':'invoiced','invoiced':True})
        state=self.browse(cr,uid,ids,context=context)[0].state
        objs=self.browse(cr,uid,ids)
        self._history(cr, uid, objs, _(state or ''))
        self.log(cr, uid,ids[0], "Quotation invoiced!")
        
        return True
    
    def to_done(self,cr,uid,ids,context=None):
        
       
        self.write(cr,uid,ids,{'state':'done'})
        state=self.browse(cr,uid,ids,context=context)[0].state
        objs=self.browse(cr,uid,ids)
        self._history(cr, uid, objs, _(state or ''))
        self.log(cr, uid,ids[0], "Quotation done!")
        
        return True
    
    def _history(self, cr, uid, cases, keyword, history=False, subject=None, email=False, details=None, email_from=False, message_id=False, attach=[], context=None):
        mailgate_pool = self.pool.get('mailgate.thread')
        return mailgate_pool.history(cr, uid, cases, keyword, history=history,\
                                       subject=subject, email=email, \
                                       details=details, email_from=email_from,\
                                       message_id=message_id, attach=attach, \
                                       context=context)
    
    def create(self, cr, uid, vals, context=None):
        
        customer=self.pool.get('res.partner').browse(cr,uid,[vals['cust_id']])[0].name.upper()
        location=self.pool.get('res.country.state').browse(cr,uid,[vals['city_id']])[0].name.upper()
        reception_date=str(vals['recp_date']).replace('-','')
        participant=str(vals['participant'])
        order_no=customer+"_"+location+"_"+participant
        
        vals['name']=order_no
        
        return super(EventOrder, self).create(cr,uid, vals,context=context)
    
    def write(self, cr, uid, ids, vals, context=None):
        objs=self.browse(cr,uid,ids)
        objs_read=self.read(cr,uid,ids)
       
        for each in objs:
         
            if each.state=='invoiced':
                 
                 if not each.invoice: 
                     number = self.pool.get('ir.sequence').search(cr, uid, [('code','=','event.invoice')]),
                     sequence = self.pool.get('ir.sequence').get_id(cr, uid, number[0][0])
                     vals.update({'invoice':sequence or ''})

        return super(EventOrder, self).write(cr, uid, ids, vals, context=context)
    
    
EventOrder()




class EventOrderLine(osv.osv):
    
    UNIT_PRICE=0
    
    def _amount_all(self,cr,uid,ids,name,args,context=None):
        
        res={}
        val=0
        quantity=1

        for each in self.browse(cr, uid, ids, context=context):
            res[each.id] = { 'amount_untaxed': 0.0,'amount_total': 0.0,'amount_taxed':0.0,'discounted_amount':0.0,}
            for line in each.service_ids:
        
                res[each.id]['amount_taxed'] += (line.total or 0.0)*(float(line.vat) or 0.0)/100
                res[each.id]['amount_untaxed']+=line.total
            
            quantity=each.quantity
           
            res[each.id]['amount_taxed']=res[each.id]['amount_taxed']*quantity
            res[each.id]['amount_untaxed']=res[each.id]['amount_untaxed']*quantity    
            discount=self.browse(cr,uid,ids)[0].discount
            res[each.id]['discounted_amount'] = (res[each.id]['amount_taxed'] + res[each.id]['amount_untaxed'] )*(discount or 0.0)/100
            res[each.id]['amount_total'] = (res[each.id]['amount_taxed'] + res[each.id]['amount_untaxed'] - res[each.id]['discounted_amount'])

        return res
    
        
    
    _name="event.order.line"
    
    _columns={
              'name':fields.char('Description',size=128,required=True),
              'product_id':fields.many2one('event.product','Product'),
              'discount':fields.float('Discount(%)',digits=(16,2)),
              'quantity':fields.integer('Quantity'),
              'day':fields.date('Date',required=True),
              'price_unit':fields.float('Unit Price',digits=(16,2)),
              'service_ids': fields.many2many('product.service','order_service_quan_rel','service_ids','name','Services'), 
              'order_id':fields.many2one('event.quotation','Quaotation'),
              'amount_untaxed':fields.function(_amount_all,method=True,multi=True,store=True,string='Amount Untaxed',digits=(16,2)),
              'amount_total':fields.function(_amount_all,method=True,multi=True,store=True,string='Amount Total',digits=(16,2)),
              'amount_taxed':fields.function(_amount_all,method=True,multi=True,store=True,string='Amount Taxed',digits=(16,2)),
              'discounted_amount':fields.function(_amount_all,method=True,multi=True,store=True,string='Discount Amount',digits=(16,2)),
              'state':fields.selection([('draft','Draft'),('done','Done')],'State')
              }
    
    _defaults={
               'discount': lambda *a:0.0,
               'state': 'draft',
               'quantity': 1,
               }
    
    def product_id_change(self,cr,uid,ids,product_id,partner_id=False,package_id=False,context=None):
       
        if partner_id:
            lang = self.pool.get('res.partner').browse(cr, uid, partner_id).lang
            context = {'lang': lang, 'partner_id': partner_id}
                 
        result = {}
        
        service_object=self.pool.get('product.service')
  
        unit_price=[]

        if product_id:
            
            pro_obj=self.pool.get('event.product').browse(cr,uid,[product_id])
         
            for each in pro_obj:
                ser_ids=[id.id for id in each.service_quan_ids]
                
            for each in service_object.browse(cr,uid,ser_ids):
                unit_price.append(each.unit_buy*each.quantity)
                
            result['price_unit']=sum(unit_price or [])        
            result['service_ids']=ser_ids
            result['name']=self.pool.get('event.product').name_get(cr,uid,[product_id],context=context)[0][1]
           
    
        return {'value': result}
    
    
    def onchange_service(self,cr,uid,ids,product_id,service_ids,context=None):
        
        res={}
        unit_price=[]
        warning={}
       
        if not service_ids[0][2]:
            warning = { 'title': _('Service Information !'),'message': _("Please save the record before modifying it!"),}
 
 
#        self.product_id_change(cr,uid,ids,product_id)
        service_object=self.pool.get('product.service')
        
        for each in service_object.browse(cr,uid,service_ids[0][2]):
                unit_price.append(each.unit_buy*each.quantity)
                            
        res['price_unit']=sum(unit_price or [])        
       
        return {'value':res,'warning':warning}
    
    
    def compute(self,cr,uid,ids,context=None):
        
        if ids:
            unit_price=[]
            service_object=self.pool.get('product.service')
            pro_obj=self.pool.get('event.product')
                        
            for each in self.browse(cr,uid,ids):
                ser_ids=[id.id for id in each.product_id.service_quan_ids]
                self.product_id_change(cr,uid,ids,each.product_id.id)
            for each in service_object.browse(cr,uid,ser_ids):
                unit_price.append(each.unit_buy*each.quantity)
            
            self.write(cr,uid,ids,{'price_unit':sum(unit_price or [])})    
        
        return True
    
    

EventOrderLine()






